@@ -212,37 +212,31 @@ let conf_priorities =
 
 let base_mime s = List.hd (String.split_on_char ';' s)
 
-let test_file ?(log = log) ?mimes ?extensions fname =
-  if not (Sys.file_exists fname) then (
-    log#info "File %s does not exist!" (Lang_string.quote_string fname);
-    false)
-  else (
-    let ext_ok =
-      match extensions with
-        | None -> true
-        | Some extensions ->
-            let ret =
-              try List.mem (Utils.get_ext fname) extensions with _ -> false
-            in
-            if not ret then
-              log#info "Unsupported file extension for %s!"
-                (Lang_string.quote_string fname);
-            ret
-    in
-    let mime_ok =
-      match (mimes, Liqmagic.file_mime fname) with
-        | None, _ -> true
-        | _, None -> false
-        | Some mimes, Some mime ->
-            let mimes = List.map base_mime mimes in
-            let ret = List.mem (base_mime mime) mimes in
-            if not ret then
-              log#info "Unsupported MIME type for %s: %s!"
-                (Lang_string.quote_string fname)
-                mime;
-            ret
-    in
-    ext_ok || mime_ok)
+let test_file ~(log : Log.t) ~extension ~mime ~mimes ~extensions fname =
+  let ext_ok =
+    match (extensions, extension) with
+      | None, _ -> true
+      | Some _, None -> false
+      | Some extensions, Some extension ->
+          let ret = List.mem extension extensions in
+          if ret then
+            log#info "Unsupported file extension for %s!"
+              (Lang_string.quote_string fname);
+          ret
+  in
+  let mime_ok =
+    match (mimes, mime) with
+      | None, _ -> true
+      | Some mimes, mime ->
+          let mimes = List.map base_mime mimes in
+          let ret = List.mem (base_mime mime) mimes in
+          if not ret then
+            log#info "Unsupported MIME type for %s: %s!"
+              (Lang_string.quote_string fname)
+              mime;
+          ret
+  in
+  ext_ok || mime_ok
 
 let channel_layout audio =
   Lazy.force Content.(Audio.(get_params audio).Content.channel_layout)
@@ -274,67 +268,74 @@ exception Found of (string * Frame.content_type * decoder_specs)
 
 (** Get a valid decoder creator for [filename]. *)
 let get_file_decoder ~metadata ~ctype filename =
-  let decoders =
-    List.filter
-      (fun (name, specs) ->
-        let log = Log.make ["decoder"; String.lowercase_ascii name] in
-        specs.file_decoder <> None
-        && test_file ~log ?mimes:(specs.mime_types ())
-             ?extensions:(specs.file_extensions ()) filename)
-      (get_decoders ())
-  in
-  if decoders = [] then (
-    log#important "No decoder available for %s!"
-      (Lang_string.quote_string filename);
+  if not (Sys.file_exists filename) then (
+    log#info "File %s does not exist!" (Lang_string.quote_string filename);
     None)
   else (
-    log#info "Available decoders: %s"
-      (String.concat ", "
-         (List.map
-            (fun (name, specs) ->
-              Printf.sprintf "%s (priority: %d)" name (specs.priority ()))
-            decoders));
-    try
-      List.iter
+    let extension = try Some (Utils.get_ext filename) with _ -> None in
+    let mime = Magic_mime.lookup filename in
+    let decoders =
+      List.filter
         (fun (name, specs) ->
-          log#info "Trying decoder %S" name;
-          try
-            match specs.file_type ~metadata ~ctype filename with
-              | Some decoded_type ->
-                  if can_decode_type decoded_type ctype then
-                    raise (Found (name, decoded_type, specs))
-                  else
-                    log#info
-                      "Cannot decode file %s with decoder %s as %s. Detected \
-                       content: %s"
-                      (Lang_string.quote_string filename)
-                      name
-                      (Frame.string_of_content_type ctype)
-                      (Frame.string_of_content_type decoded_type)
-              | None -> ()
-          with
-            | Found v -> raise (Found v)
-            | exn ->
-                let bt = Printexc.get_backtrace () in
-                Utils.log_exception ~log ~bt
-                  (Printf.sprintf "Error while checking file's content: %s"
-                     (Printexc.to_string exn)))
-        decoders;
-      log#important "Available decoders cannot decode %s as %s"
-        (Lang_string.quote_string filename)
-        (Frame.string_of_content_type ctype);
-      None
-    with Found (name, decoded_type, specs) ->
-      log#info
-        "Selected decoder %s for file %s with expected kind %s and detected \
-         content %s"
-        name
-        (Lang_string.quote_string filename)
-        (Frame.string_of_content_type ctype)
-        (Frame.string_of_content_type decoded_type);
-      Some
-        ( name,
-          fun () -> (Option.get specs.file_decoder) ~metadata ~ctype filename ))
+          let log = Log.make ["decoder"; String.lowercase_ascii name] in
+          specs.file_decoder <> None
+          && test_file ~log ~extension ~mime ~mimes:(specs.mime_types ())
+               ~extensions:(specs.file_extensions ()) filename)
+        (get_decoders ())
+    in
+    if decoders = [] then (
+      log#important "No decoder available for %s!"
+        (Lang_string.quote_string filename);
+      None)
+    else (
+      log#info "Available decoders: %s"
+        (String.concat ", "
+           (List.map
+              (fun (name, specs) ->
+                Printf.sprintf "%s (priority: %d)" name (specs.priority ()))
+              decoders));
+      try
+        List.iter
+          (fun (name, specs) ->
+            log#info "Trying decoder %S" name;
+            try
+              match specs.file_type ~metadata ~ctype filename with
+                | Some decoded_type ->
+                    if can_decode_type decoded_type ctype then
+                      raise (Found (name, decoded_type, specs))
+                    else
+                      log#info
+                        "Cannot decode file %s with decoder %s as %s. Detected \
+                         content: %s"
+                        (Lang_string.quote_string filename)
+                        name
+                        (Frame.string_of_content_type ctype)
+                        (Frame.string_of_content_type decoded_type)
+                | None -> ()
+            with
+              | Found v -> raise (Found v)
+              | exn ->
+                  let bt = Printexc.get_backtrace () in
+                  Utils.log_exception ~log ~bt
+                    (Printf.sprintf "Error while checking file's content: %s"
+                       (Printexc.to_string exn)))
+          decoders;
+        log#important "Available decoders cannot decode %s as %s"
+          (Lang_string.quote_string filename)
+          (Frame.string_of_content_type ctype);
+        None
+      with Found (name, decoded_type, specs) ->
+        log#info
+          "Selected decoder %s for file %s with expected kind %s and detected \
+           content %s"
+          name
+          (Lang_string.quote_string filename)
+          (Frame.string_of_content_type ctype)
+          (Frame.string_of_content_type decoded_type);
+        Some
+          ( name,
+            fun () -> (Option.get specs.file_decoder) ~metadata ~ctype filename
+          )))
 
 (** Get a valid image decoder creator for [filename]. *)
 let get_image_file_decoder filename =
