@@ -24,6 +24,7 @@
 
 exception End_of_file
 exception No_stream
+exception Invalid_file
 
 let log = Log.make ["decoder"; "ffmpeg"]
 
@@ -562,26 +563,40 @@ let () =
 
 let tags_substitutions = [("track", "tracknumber")]
 
-let get_tags ~metadata file =
-  let args, format = parse_file_decoder_args metadata in
-  let opts = Hashtbl.create 10 in
-  List.iter (fun (k, v) -> Hashtbl.add opts k v) args;
-  let container = Av.open_input ?format ~opts file in
-  Fun.protect
-    ~finally:(fun () -> Av.close container)
-    (fun () ->
-      (* For now we only add the metadata from the best audio track *)
-      let audio_tags =
-        try
-          let _, s, _ = Av.find_best_audio_stream container in
-          Av.get_metadata s
-        with _ -> []
-      in
-      let tags = Av.get_input_metadata container in
-      List.map
-        (fun (lbl, v) ->
-          try (List.assoc lbl tags_substitutions, v) with _ -> (lbl, v))
-        (audio_tags @ tags))
+let get_tags ~metadata ~extension ~mime file =
+  try
+    if
+      not
+        (Decoder.test_file ~log ~extension ~mime ~mimes:(Some mime_types#get)
+           ~extensions:(Some file_extensions#get) file)
+    then raise Invalid_file;
+    let args, format = parse_file_decoder_args metadata in
+    let opts = Hashtbl.create 10 in
+    List.iter (fun (k, v) -> Hashtbl.add opts k v) args;
+    let container = Av.open_input ?format ~opts file in
+    Fun.protect
+      ~finally:(fun () -> Av.close container)
+      (fun () ->
+        (* For now we only add the metadata from the best audio track *)
+        let audio_tags =
+          try
+            let _, s, _ = Av.find_best_audio_stream container in
+            Av.get_metadata s
+          with _ -> []
+        in
+        let tags = Av.get_input_metadata container in
+        List.map
+          (fun (lbl, v) ->
+            try (List.assoc lbl tags_substitutions, v) with _ -> (lbl, v))
+          (audio_tags @ tags))
+  with
+    | Invalid_file -> []
+    | e ->
+        let bt = Printexc.get_backtrace () in
+        Utils.log_exception ~log ~bt
+          (Printf.sprintf "Error while decoding file tags: %s"
+             (Printexc.to_string e));
+        raise Not_found
 
 let () = Plug.register Request.mresolvers "ffmpeg" ~doc:"" get_tags
 
@@ -754,13 +769,45 @@ let mk_eof streams buffer =
     streams
 
 let mk_decoder ~streams ~target_position container =
-  let check_pts stream pts =
+  let streams_seen = Hashtbl.create 0 in
+  let position ~pts stream =
+    let { Avutil.num; den } = Av.get_time_base stream in
+    Int64.to_float pts *. float num /. float den
+  in
+  let decodable = ref [] in
+  let push (position, decode) =
+    decodable :=
+      (position, decode)
+      :: List.filter
+           (fun (p, _) ->
+             Float.abs (p -. position)
+             <= Ffmpeg_decoder_common.conf_max_interleave_duration#get)
+           !decodable
+  in
+  let flush position =
+    let d = List.sort (fun (p, _) (p', _) -> Float.compare p p') !decodable in
+    let min_position =
+      position -. Ffmpeg_decoder_common.conf_max_interleave_delta#get
+    in
+    List.iter (fun (p, decode) -> if min_position <= p then decode ()) d;
+    decodable := []
+  in
+  let check_pts ~decode stream pts =
     match (pts, !target_position) with
       | Some pts, Some target_position ->
-          let { Avutil.num; den } = Av.get_time_base stream in
-          let position = Int64.to_float pts *. float num /. float den in
-          target_position <= position
-      | _ -> true
+          if target_position <= position ~pts stream then decode ()
+      | Some pts, None ->
+          Hashtbl.replace streams_seen (Hashtbl.hash stream) true;
+          let position = position ~pts stream in
+          if Hashtbl.length streams_seen = Streams.cardinal streams then (
+            flush position;
+            decode ())
+          else push (position, decode)
+      | None, _ ->
+          log#important
+            "Got packet or frame with no timestamp! Synchronization issues may \
+             happen.";
+          decode ()
   in
   let audio_frame =
     Streams.fold
@@ -800,32 +847,40 @@ let mk_decoder ~streams ~target_position container =
           | `Audio_frame (i, frame) -> (
               match Streams.find_opt i streams with
                 | Some (`Audio_frame (s, decode)) ->
-                    if check_pts s (Avutil.Frame.pts frame) then
-                      decode ~buffer (`Frame frame)
+                    check_pts s
+                      ~decode:(fun () -> decode ~buffer (`Frame frame))
+                      (Avutil.Frame.pts frame)
                 | _ -> f ())
           | `Audio_packet (i, packet) -> (
               match Streams.find_opt i streams with
                 | Some (`Audio_packet (s, decode)) ->
-                    if check_pts s (Avcodec.Packet.get_pts packet) then
-                      decode ~buffer packet
+                    check_pts
+                      ~decode:(fun () -> decode ~buffer packet)
+                      s
+                      (Avcodec.Packet.get_pts packet)
                 | _ -> f ())
           | `Video_frame (i, frame) -> (
               match Streams.find_opt i streams with
                 | Some (`Video_frame (s, decode)) ->
-                    if check_pts s (Avutil.Frame.pts frame) then
-                      decode ~buffer (`Frame frame)
+                    check_pts
+                      ~decode:(fun () -> decode ~buffer (`Frame frame))
+                      s (Avutil.Frame.pts frame)
                 | _ -> f ())
           | `Video_packet (i, packet) -> (
               match Streams.find_opt i streams with
                 | Some (`Video_packet (s, decode)) ->
-                    if check_pts s (Avcodec.Packet.get_pts packet) then
-                      decode ~buffer packet
+                    check_pts
+                      ~decode:(fun () -> decode ~buffer packet)
+                      s
+                      (Avcodec.Packet.get_pts packet)
                 | _ -> f ())
           | `Data_packet (i, packet) -> (
               match Streams.find_opt i streams with
                 | Some (`Data_packet (s, decode)) ->
-                    if check_pts s (Avcodec.Packet.get_pts packet) then
-                      decode ~buffer packet
+                    check_pts
+                      ~decode:(fun () -> decode ~buffer packet)
+                      s
+                      (Avcodec.Packet.get_pts packet)
                 | _ -> f ())
           | _ -> ()
       with
