@@ -1014,7 +1014,9 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
     let server = transport#server in
     let process_client sock =
       try
-        let socket, caller = server#accept sock in
+        let socket, caller =
+          server#accept ?timeout:(Some conf_accept_timeout#get) sock
+        in
         let ip = Utils.name_of_sockaddr ~rev_dns:conf_revdns#get caller in
         log#info "New client on port %i: %s" port ip;
         let unix_socket = T.file_descr_of_socket socket in
@@ -1067,7 +1069,10 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
         in
         Duppy.Monad.run ~return:reply ~raise:reply (handle_client ~port ~icy h)
       with e ->
-        log#severe "Failed to accept new client: %s" (Printexc.to_string e)
+        let bt = Printexc.get_backtrace () in
+        Utils.log_exception ~log ~bt
+          (Printf.sprintf "Failed to accept new client: %s"
+             (Printexc.to_string e))
     in
     let rec incoming ~port ~icy events (out_s : Unix.file_descr) e =
       if List.mem (`Read out_s) e then (
