@@ -54,6 +54,12 @@ type container = {
   closed : bool Atomic.t;
 }
 
+let shutdown = Atomic.make false
+
+let () =
+  Lifecycle.before_core_shutdown ~name:"input.ffmpeg shutdown" (fun () ->
+      Atomic.set shutdown true)
+
 class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
   ~clock_safe ~max_buffer ~on_error ~on_stop ~on_start ~on_connect
   ~metadata_filter ~on_disconnect ~new_track_on_metadata ?format ~opts ~trim_url
@@ -64,16 +70,8 @@ class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
       Start_stop.active_source
         ~name ~fallible:true ~clock_safe ~on_start ~on_stop ~autostart () as super
 
-    inherit Source.no_seek
     val connect_task = Atomic.make None
     val container = Atomic.make None
-    val shutdown = Atomic.make false
-
-    initializer
-      Lifecycle.before_core_shutdown
-        ~name:(Printf.sprintf "%s shutdown" self#id) (fun () ->
-          Atomic.set shutdown true)
-
     method seek_source = (self :> Source.source)
     method remaining = -1
     method abort_track = Generator.add_track_mark self#buffer
