@@ -385,18 +385,23 @@ let mk_socket ~payload_size ~messageapi () =
   s
 
 let close_socket s = Srt.close s
+let shutdown = Atomic.make false
+
+let () =
+  Lifecycle.on_core_shutdown ~name:"srt shutdown" (fun () ->
+      Atomic.set shutdown true)
+
+let id =
+  let counter = Atomic.make 0 in
+  fun () -> Atomic.fetch_and_add counter 1
 
 class virtual base =
-  object (self)
-    method virtual id : string
-    method virtual mutexify : 'a 'b. ('a -> 'b) -> 'a -> 'b
-    val mutable should_stop = false
-    method private should_stop = self#mutexify (fun () -> should_stop) ()
-    method private set_should_stop = self#mutexify (fun b -> should_stop <- b)
-
-    initializer
-      Lifecycle.on_core_shutdown ~name:(Printf.sprintf "%s shutdown" self#id)
-        (fun () -> self#set_should_stop true)
+  object
+    val should_stop = Atomic.make false
+    val id = id ()
+    method private should_stop = Atomic.get shutdown || Atomic.get should_stop
+    method private set_should_stop = Atomic.set should_stop
+    method srt_id = id
   end
 
 class virtual networking_agent =
@@ -423,6 +428,18 @@ class virtual output_networking_agent =
         : Srt.socket -> exn -> Printexc.raw_backtrace -> unit
   end
 
+module ToDisconnect = Liquidsoap_lang.Active_value.Make (struct
+  type t = < disconnect : unit ; srt_id : int >
+
+  let id t = t#srt_id
+end)
+
+let to_disconnect = ToDisconnect.create 10
+
+let () =
+  Lifecycle.on_core_shutdown ~name:"Srt disconnect" (fun () ->
+      ToDisconnect.iter (fun s -> s#disconnect) to_disconnect)
+
 class virtual caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
   ~polling_delay ~payload_size ~messageapi ~hostname ~port ~connection_timeout
   ~read_timeout ~write_timeout ~on_connect ~on_disconnect =
@@ -430,101 +447,83 @@ class virtual caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
     method virtual id : string
     method virtual should_stop : bool
     val mutable connect_task = None
-    val mutable task_should_stop = false
-    val mutable socket = None
-
-    initializer
-      Lifecycle.on_core_shutdown ~name:(Printf.sprintf "%s shutdown" self#id)
-        (fun () -> self#disconnect)
+    val task_should_stop = Atomic.make false
+    val socket = Atomic.make None
+    initializer ToDisconnect.add to_disconnect (self :> ToDisconnect.data)
 
     method private get_socket =
-      self#mutexify
-        (fun () ->
-          match socket with Some s -> s | None -> raise Not_connected)
-        ()
+      match Atomic.get socket with Some s -> s | None -> raise Not_connected
 
     method virtual private log : Log.t
-    method virtual private mutexify : 'a 'b. ('a -> 'b) -> 'a -> 'b
-    method private is_connected = self#mutexify (fun () -> socket <> None) ()
+    method private is_connected = Atomic.get socket <> None
 
     method private connect_fn () =
-      self#mutexify
-        (fun () ->
-          try
-            let ipaddr = (Unix.gethostbyname hostname).Unix.h_addr_list.(0) in
-            let sockaddr = Unix.ADDR_INET (ipaddr, port) in
-            self#log#important "Connecting to srt://%s:%d.." hostname port;
-            ignore (Option.map (fun (_, s) -> close_socket s) socket);
-            let s = mk_socket ~payload_size ~messageapi () in
-            Srt.setsockflag s Srt.sndsyn true;
-            Srt.setsockflag s Srt.rcvsyn true;
-            ignore
-              (Option.map (fun id -> Srt.(setsockflag s streamid id)) streamid);
-            ignore
-              (Option.map
-                 (fun b -> Srt.(setsockflag s enforced_encryption b))
-                 enforced_encryption);
-            ignore
-              (Option.map
-                 (fun len -> Srt.(setsockflag s pbkeylen len))
-                 pbkeylen);
-            ignore
-              (Option.map
-                 (fun p -> Srt.(setsockflag s passphrase p))
-                 passphrase);
-            ignore
-              (Option.map
-                 (fun v -> Srt.(setsockflag s conntimeo v))
-                 connection_timeout);
-            ignore
-              (Option.map
-                 (fun v -> Srt.(setsockflag s sndtimeo v))
-                 write_timeout);
-            ignore
-              (Option.map
-                 (fun v -> Srt.(setsockflag s rcvtimeo v))
-                 read_timeout);
-            Srt.connect s sockaddr;
-            socket <- Some (sockaddr, s);
-            self#log#important "Client connected!";
-            !on_connect ();
-            -1.
-          with exn ->
-            self#log#important "Connect failed: %s" (Printexc.to_string exn);
-            if not task_should_stop then polling_delay else -1.)
-        ()
+      try
+        let ipaddr = (Unix.gethostbyname hostname).Unix.h_addr_list.(0) in
+        let sockaddr = Unix.ADDR_INET (ipaddr, port) in
+        self#log#important "Connecting to srt://%s:%d.." hostname port;
+        (match Atomic.exchange socket None with
+          | None -> ()
+          | Some (_, s) -> close_socket s);
+        let s = mk_socket ~payload_size ~messageapi () in
+        try
+          Srt.setsockflag s Srt.sndsyn true;
+          Srt.setsockflag s Srt.rcvsyn true;
+          ignore
+            (Option.map (fun id -> Srt.(setsockflag s streamid id)) streamid);
+          ignore
+            (Option.map
+               (fun b -> Srt.(setsockflag s enforced_encryption b))
+               enforced_encryption);
+          ignore
+            (Option.map (fun len -> Srt.(setsockflag s pbkeylen len)) pbkeylen);
+          ignore
+            (Option.map (fun p -> Srt.(setsockflag s passphrase p)) passphrase);
+          ignore
+            (Option.map
+               (fun v -> Srt.(setsockflag s conntimeo v))
+               connection_timeout);
+          ignore
+            (Option.map (fun v -> Srt.(setsockflag s sndtimeo v)) write_timeout);
+          ignore
+            (Option.map (fun v -> Srt.(setsockflag s rcvtimeo v)) read_timeout);
+          Srt.connect s sockaddr;
+          Atomic.set socket (Some (sockaddr, s));
+          self#log#important "Client connected!";
+          !on_connect ();
+          -1.
+        with exn ->
+          let bt = Printexc.get_raw_backtrace () in
+          Srt.close s;
+          Printexc.raise_with_backtrace exn bt
+      with exn ->
+        self#log#important "Connect failed: %s" (Printexc.to_string exn);
+        if not (Atomic.get task_should_stop) then polling_delay else -1.
 
     method private connect =
-      self#mutexify
-        (fun () ->
-          task_should_stop <- false;
-          match connect_task with
-            | Some t -> Duppy.Async.wake_up t
-            | None ->
-                let t =
-                  Duppy.Async.add ~priority:`Blocking Tutils.scheduler
-                    self#connect_fn
-                in
-                connect_task <- Some t;
-                Duppy.Async.wake_up t)
-        ()
+      Atomic.set task_should_stop false;
+      match connect_task with
+        | Some t -> Duppy.Async.wake_up t
+        | None ->
+            let t =
+              Duppy.Async.add ~priority:`Blocking Tutils.scheduler
+                self#connect_fn
+            in
+            connect_task <- Some t;
+            Duppy.Async.wake_up t
 
-    method private disconnect =
-      self#mutexify
-        (fun () ->
-          (match socket with
-            | None -> ()
-            | Some (_, socket) ->
-                close_socket socket;
-                !on_disconnect ());
-          socket <- None;
-          task_should_stop <- true;
-          match connect_task with
-            | None -> ()
-            | Some t ->
-                Duppy.Async.stop t;
-                connect_task <- None)
-        ()
+    method disconnect =
+      (match Atomic.exchange socket None with
+        | None -> ()
+        | Some (_, socket) ->
+            close_socket socket;
+            !on_disconnect ());
+      Atomic.set task_should_stop true;
+      match connect_task with
+        | None -> ()
+        | Some t ->
+            Duppy.Async.stop t;
+            connect_task <- None
   end
 
 class virtual listener ~enforced_encryption ~pbkeylen ~passphrase ~max_clients
@@ -537,10 +536,7 @@ class virtual listener ~enforced_encryption ~pbkeylen ~passphrase ~max_clients
     method virtual should_stop : bool
     method virtual mutexify : 'a 'b. ('a -> 'b) -> 'a -> 'b
     val listening_socket = Atomic.make None
-
-    initializer
-      Lifecycle.on_core_shutdown ~name:(Printf.sprintf "%s shutdown" self#id)
-        (fun () -> self#disconnect)
+    initializer ToDisconnect.add to_disconnect (self :> ToDisconnect.data)
 
     method private is_connected =
       self#mutexify (fun () -> client_sockets <> []) ()
@@ -629,7 +625,7 @@ class virtual listener ~enforced_encryption ~pbkeylen ~passphrase ~max_clients
             Poll.add_socket ~mode:`Read self#listening_socket accept_connection)
           ()
 
-    method private disconnect =
+    method disconnect =
       let should_stop = self#should_stop in
       self#mutexify
         (fun () ->
@@ -647,7 +643,7 @@ class virtual listener ~enforced_encryption ~pbkeylen ~passphrase ~max_clients
         ()
   end
 
-class virtual input_base ~max ~clock_safe ~on_connect ~on_disconnect
+class virtual input_base ~max ~self_sync ~clock_safe ~on_connect ~on_disconnect
   ~payload_size ~dump ~on_start ~on_stop ~autostart format =
   let max_length = Some (Frame.main_of_seconds max) in
   object (self)
@@ -685,7 +681,6 @@ class virtual input_base ~max ~clock_safe ~on_connect ~on_disconnect
       super#wake_up act;
       Generator.set_max_length self#buffer max_length
 
-    method seek _ = 0
     method seek_source = (self :> Source.source)
     method remaining = -1
     method abort_track = Generator.add_track_mark self#buffer
@@ -693,7 +688,8 @@ class virtual input_base ~max ~clock_safe ~on_connect ~on_disconnect
     method! is_ready ?frame () =
       super#is_ready ?frame () && (not self#should_stop) && self#is_connected
 
-    method self_sync = (`Dynamic, self#is_connected)
+    method self_sync =
+      if self_sync then (`Dynamic, self#is_connected) else (`Static, false)
 
     method private create_decoder socket =
       let create_decoder =
@@ -760,14 +756,14 @@ class virtual input_base ~max ~clock_safe ~on_connect ~on_disconnect
   end
 
 class input_listener ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
-  ~bind_address ~max ~payload_size ~clock_safe ~on_connect ~on_disconnect
-  ~read_timeout ~write_timeout ~messageapi ~dump ~on_start ~on_stop ~autostart
-  format =
+  ~bind_address ~max ~payload_size ~self_sync ~clock_safe ~on_connect
+  ~on_disconnect ~read_timeout ~write_timeout ~messageapi ~dump ~on_start
+  ~on_stop ~autostart format =
   object (self)
     inherit
       input_base
-        ~max ~payload_size ~clock_safe ~on_connect ~on_disconnect ~dump
-          ~on_start ~on_stop ~autostart format
+        ~max ~payload_size ~self_sync ~clock_safe ~on_connect ~on_disconnect
+          ~dump ~on_start ~on_stop ~autostart format
 
     inherit
       listener
@@ -783,14 +779,14 @@ class input_listener ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
   end
 
 class input_caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
-  ~polling_delay ~hostname ~port ~max ~payload_size ~clock_safe ~on_connect
-  ~on_disconnect ~read_timeout ~write_timeout ~connection_timeout ~messageapi
-  ~dump ~on_start ~on_stop ~autostart format =
+  ~polling_delay ~hostname ~port ~max ~payload_size ~self_sync ~clock_safe
+  ~on_connect ~on_disconnect ~read_timeout ~write_timeout ~connection_timeout
+  ~messageapi ~dump ~on_start ~on_stop ~autostart format =
   object (self)
     inherit
       input_base
-        ~max ~payload_size ~clock_safe ~on_connect ~on_disconnect ~dump
-          ~on_start ~on_stop ~autostart format
+        ~max ~payload_size ~self_sync ~clock_safe ~on_connect ~on_disconnect
+          ~dump ~on_start ~on_stop ~autostart format
 
     inherit
       caller
@@ -814,6 +810,13 @@ let _ =
           Lang.float_t,
           Some (Lang.float 10.),
           Some "Maximum duration of the buffered data." );
+        ( "self_sync",
+          Lang.bool_t,
+          Some (Lang.bool true),
+          Some
+            "`true` if the source controls its own latency (i.e. the SRT \
+             stream is in `live` mode), `false` otherwise (i.e. the stream is \
+             in `file` mode." );
         ( "dump",
           Lang.string_t,
           Some (Lang.string ""),
@@ -856,6 +859,7 @@ let _ =
       in
       let max = Lang.to_float (List.assoc "max" p) in
       let clock_safe = Lang.to_bool (List.assoc "clock_safe" p) in
+      let self_sync = Lang.to_bool (List.assoc "self_sync" p) in
       let on_start =
         let f = List.assoc "on_start" p in
         fun () -> ignore (Lang.apply f [])
@@ -871,22 +875,23 @@ let _ =
             (new input_listener
                ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
                ~bind_address ~read_timeout ~write_timeout ~payload_size
-               ~clock_safe ~on_connect ~on_disconnect ~messageapi ~max ~dump
-               ~on_start ~on_stop ~autostart format
+               ~self_sync ~clock_safe ~on_connect ~on_disconnect ~messageapi
+               ~max ~dump ~on_start ~on_stop ~autostart format
               :> < Start_stop.active_source
                  ; get_sockets : (Unix.sockaddr * Srt.socket) list >)
         | `Caller ->
             (new input_caller
                ~enforced_encryption ~pbkeylen ~passphrase ~streamid
-               ~polling_delay ~hostname ~port ~payload_size ~clock_safe
-               ~on_connect ~read_timeout ~write_timeout ~connection_timeout
-               ~on_disconnect ~messageapi ~max ~dump ~on_start ~on_stop
-               ~autostart format
+               ~polling_delay ~hostname ~port ~payload_size ~self_sync
+               ~clock_safe ~on_connect ~read_timeout ~write_timeout
+               ~connection_timeout ~on_disconnect ~messageapi ~max ~dump
+               ~on_start ~on_stop ~autostart format
               :> < Start_stop.active_source
                  ; get_sockets : (Unix.sockaddr * Srt.socket) list >))
 
 class virtual output_base ~payload_size ~messageapi ~on_start ~on_stop
-  ~infallible ~autostart ~on_disconnect ~encoder_factory source =
+  ~infallible ~register_telnet ~autostart ~on_disconnect ~encoder_factory source
+  =
   let buffer = Strings.Mutable.empty () in
   let tmp = Bytes.create payload_size in
   object (self)
@@ -895,8 +900,8 @@ class virtual output_base ~payload_size ~messageapi ~on_start ~on_stop
 
     inherit
       Output.encoded
-        ~output_kind:"srt" ~on_start ~on_stop ~infallible ~autostart
-          ~name:"output.srt" source
+        ~output_kind:"srt" ~on_start ~on_stop ~infallible ~register_telnet
+          ~export_cover_metadata:false ~autostart ~name:"output.srt" source
 
     val mutable encoder = None
 
@@ -953,7 +958,7 @@ class virtual output_base ~payload_size ~messageapi ~on_start ~on_stop
         ()
 
     method private start =
-      self#mutexify (fun () -> should_stop <- false) ();
+      self#mutexify (fun () -> Atomic.set should_stop false) ();
       self#connect
 
     method! private reset =
@@ -961,7 +966,7 @@ class virtual output_base ~payload_size ~messageapi ~on_start ~on_stop
       self#stop
 
     method private stop =
-      self#mutexify (fun () -> should_stop <- true) ();
+      self#mutexify (fun () -> Atomic.set should_stop true) ();
       self#disconnect
 
     method private encode frame ofs len =
@@ -979,13 +984,13 @@ class virtual output_base ~payload_size ~messageapi ~on_start ~on_stop
 
 class output_caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
   ~polling_delay ~payload_size ~messageapi ~on_start ~on_stop ~infallible
-  ~autostart ~on_connect ~on_disconnect ~port ~hostname ~read_timeout
-  ~write_timeout ~connection_timeout ~encoder_factory source =
+  ~register_telnet ~autostart ~on_connect ~on_disconnect ~port ~hostname
+  ~read_timeout ~write_timeout ~connection_timeout ~encoder_factory source =
   object (self)
     inherit
       output_base
-        ~payload_size ~messageapi ~on_start ~on_stop ~infallible ~autostart
-          ~on_disconnect ~encoder_factory source
+        ~payload_size ~messageapi ~on_start ~on_stop ~infallible
+          ~register_telnet ~autostart ~on_disconnect ~encoder_factory source
 
     inherit
       caller
@@ -1007,13 +1012,13 @@ class output_caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
 
 class output_listener ~enforced_encryption ~pbkeylen ~passphrase
   ~listen_callback ~max_clients ~payload_size ~messageapi ~on_start ~on_stop
-  ~infallible ~autostart ~on_connect ~on_disconnect ~bind_address ~read_timeout
-  ~write_timeout ~encoder_factory source =
+  ~infallible ~register_telnet ~autostart ~on_connect ~on_disconnect
+  ~bind_address ~read_timeout ~write_timeout ~encoder_factory source =
   object (self)
     inherit
       output_base
-        ~payload_size ~messageapi ~on_start ~on_stop ~infallible ~autostart
-          ~on_disconnect ~encoder_factory source
+        ~payload_size ~messageapi ~on_start ~on_stop ~infallible
+          ~register_telnet ~autostart ~on_disconnect ~encoder_factory source
 
     inherit
       listener
@@ -1081,6 +1086,7 @@ let _ =
         Lang.to_valued_option Lang.to_int (List.assoc "max_clients" p)
       in
       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
+      let register_telnet = Lang.to_bool (List.assoc "register_telnet" p) in
       let autostart = Lang.to_bool (List.assoc "start" p) in
       let on_start =
         let f = List.assoc "on_start" p in
@@ -1105,15 +1111,16 @@ let _ =
                ~enforced_encryption ~pbkeylen ~passphrase ~streamid
                ~polling_delay ~hostname ~port ~payload_size ~autostart ~on_start
                ~on_stop ~read_timeout ~write_timeout ~connection_timeout
-               ~infallible ~messageapi ~encoder_factory ~on_connect
-               ~on_disconnect source
+               ~infallible ~register_telnet ~messageapi ~encoder_factory
+               ~on_connect ~on_disconnect source
               :> < Output.output
                  ; get_sockets : (Unix.sockaddr * Srt.socket) list >)
         | `Listener ->
             (new output_listener
                ~enforced_encryption ~pbkeylen ~passphrase ~bind_address
                ~read_timeout ~write_timeout ~payload_size ~autostart ~on_start
-               ~on_stop ~infallible ~messageapi ~encoder_factory ~on_connect
-               ~on_disconnect ~listen_callback ~max_clients source
+               ~on_stop ~infallible ~register_telnet ~messageapi
+               ~encoder_factory ~on_connect ~on_disconnect ~listen_callback
+               ~max_clients source
               :> < Output.output
                  ; get_sockets : (Unix.sockaddr * Srt.socket) list >))
