@@ -22,6 +22,12 @@
 
 open Liquidsoap_lang.Lang
 
+module Alive_values_map = Liquidsoap_lang.Active_value.Make (struct
+  type t = Value.t
+
+  let id v = v.Value.id
+end)
+
 let log = Log.make ["lang"]
 let metadata_t = list_t (product_t string_t string_t)
 
@@ -295,15 +301,16 @@ let to_track = Track.of_value
     the currently defined source as argument). *)
 type 'a operator_method = string * scheme * string * ('a -> value)
 
+let checked_values = Alive_values_map.create 10
+
 (** Ensure that the frame contents of all the sources occurring in the value agree with [t]. *)
 let check_content v t =
-  let checked_values = ref [] in
   let check t t' = Typing.(t <: t') in
   let rec check_value v t =
-    if not (List.memq v !checked_values) then (
+    if not (Alive_values_map.mem checked_values v) then (
       (* We need to avoid checking the same value multiple times, otherwise we
          get an exponential blowup, see #1247. *)
-      checked_values := v :: !checked_values;
+      Alive_values_map.add checked_values v;
       match (v.Value.value, (Type.deref t).Type.descr) with
         | _, Type.Var _ -> ()
         | _ when Source_val.is_value v ->
@@ -489,8 +496,9 @@ let add_track_operator ~(category : Doc.Value.source) ~descr ?(flags = [])
   let category = `Track category in
   add_builtin ~category ~descr ~flags ?base name arguments return_t f
 
+let itered_values = Alive_values_map.create 10
+
 let iter_sources ?(on_imprecise = fun () -> ()) f v =
-  let itered_values = ref [] in
   let rec iter_term env v =
     let iter_base_term env v =
       match v.Term.term with
@@ -532,10 +540,10 @@ let iter_sources ?(on_imprecise = fun () -> ()) f v =
       v.Term.methods;
     iter_base_term env v
   and iter_value v =
-    if not (List.memq v !itered_values) then (
+    if not (Alive_values_map.mem itered_values v) then (
       (* We need to avoid checking the same value multiple times, otherwise we
          get an exponential blowup, see #1247. *)
-      itered_values := v :: !itered_values;
+      Alive_values_map.add itered_values v;
       Value.Methods.iter (fun _ v -> iter_value v) v.Value.methods;
       match v.value with
         | _ when Source_val.is_value v -> f (Source_val.of_value v)
