@@ -413,8 +413,7 @@ class iir (source : source) filter_family filter_type order freq1 freq2 qfactor
     (* Digital filter based on mkfilter/mkshape/gencode by A.J. Fisher *)
     method stype = source#stype
     method remaining = source#remaining
-    method seek = source#seek
-    method seek_source = source
+    method seek_source = source#seek_source
     method self_sync = source#self_sync
     method private _is_ready = source#is_ready
     method abort_track = source#abort_track
