@@ -252,12 +252,9 @@ class virtual switch ~name ~override_meta ~transition_length
         | Some s -> s.effective_source#abort_track
         | None -> ()
 
-    method seek n =
-      match selected with Some s -> s.effective_source#seek n | None -> 0
-
     method seek_source =
       match selected with
-        | Some s -> s.effective_source
+        | Some s -> s.effective_source#seek_source
         | None -> (self :> Source.source)
 
     method selected = Option.map (fun { child } -> child.source) selected
