@@ -130,7 +130,7 @@ module Buffer = struct
       method private _is_ready ?frame:_ _ =
         proceed c (fun () -> not c.buffering)
 
-      method seek len =
+      method! seek len =
         let len = min (Generator.length (Lazy.force c.generator)) len in
         Generator.truncate (Lazy.force c.generator) len;
         len
@@ -159,7 +159,8 @@ module Buffer = struct
     object
       inherit
         Output.output
-          ~output_kind:id ~infallible ~on_start ~on_stop source_val autostart
+          ~output_kind:id ~infallible ~register_telnet:false ~on_start ~on_stop
+            source_val autostart
 
       method! reset = ()
       method start = ()
@@ -299,7 +300,6 @@ module AdaptativeBuffer = struct
     let alpha = AFrame.duration () /. averaging in
     object (self)
       inherit Source.source ~name:"buffer.adaptative.producer" () as super
-      inherit Source.no_seek
       method seek_source = (self :> Source.source)
       method stype = `Fallible
       method self_sync = (`Static, false)
@@ -423,8 +423,9 @@ module AdaptativeBuffer = struct
     object (self)
       inherit
         Output.output
-          ~output_kind:"buffer" ~name:"buffer.adaptative.consumer" ~infallible
-            ~on_start ~on_stop source_val autostart
+          ~output_kind:"buffer" ~register_telnet:false
+            ~name:"buffer.adaptative.consumer" ~infallible ~on_start ~on_stop
+            source_val autostart
 
       method! reset = ()
       method start = ()
