@@ -350,6 +350,7 @@ class output p =
   let uri = Lang.Regexp.regexp [%string {|^%{uri}$|}] in
   let autostart = Lang.to_bool (List.assoc "start" p) in
   let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
+  let register_telnet = Lang.to_bool (List.assoc "register_telnet" p) in
   let on_start =
     let f = List.assoc "on_start" p in
     fun () -> ignore (Lang.apply f [])
@@ -409,8 +410,8 @@ class output p =
     (** File descriptor where to dump. *)
     inherit
       Output.encoded
-        ~output_kind:"output.harbor" ~infallible ~autostart ~on_start ~on_stop
-          ~name:mount source
+        ~output_kind:"output.harbor" ~infallible ~register_telnet ~autostart
+          ~export_cover_metadata:false ~on_start ~on_stop ~name:mount source
 
     val mutable dump = None
     val mutable encoder = None
@@ -431,7 +432,8 @@ class output p =
       Tutils.mutexify metadata.metadata_m
         (fun () -> metadata.metadata <- Some m)
         ();
-      (Option.get encoder).Encoder.insert_metadata (Export_metadata.metadata m)
+      (Option.get encoder).Encoder.insert_metadata
+        (Export_metadata.metadata ~cover:false m)
 
     method add_client ~protocol ~headers ~uri ~query s =
       let ip =
