@@ -218,6 +218,12 @@ let proto frame_t =
       ("name", Lang.nullable_t Lang.string_t, Some Lang.null, None);
       ("host", Lang.string_t, Some (Lang.string "localhost"), None);
       ("port", Lang.int_t, Some (Lang.int 8000), None);
+      ( "prefer_address",
+        Lang.nullable_t Lang.string_t,
+        Some Lang.null,
+        Some
+          "Preferred address type when resolving hostnames. One of: `\"ipv4\"` \
+           or `\"ipv6\"`. Defaults to system default when `null`." );
       ( "transport",
         Lang.http_transport_base_t,
         Some (Lang.base_http_transport Http.unix_transport),
@@ -409,6 +415,7 @@ class output p =
   in
   let autostart = Lang.to_bool (List.assoc "start" p) in
   let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
+  let register_telnet = Lang.to_bool (List.assoc "register_telnet" p) in
   let on_start =
     let f = List.assoc "on_start" p in
     fun () -> ignore (Lang.apply f [])
@@ -420,6 +427,27 @@ class output p =
   let host = s "host" in
   let port = e Lang.to_int "port" in
   let transport = e Lang.to_http_transport "transport" in
+  let prefer_address =
+    let v = List.assoc "prefer_address" p in
+    match Lang.to_valued_option Lang.to_string v with
+      | None -> `System_default
+      | Some "ipv4" -> `Ipv4
+      | Some "ipv6" -> `Ipv6
+      | Some _ ->
+          raise (Error.Invalid_value (v, "Valid values are: 'ipv4' or 'ipv6'."))
+  in
+  let transport = (transport :> Cry.transport) in
+  let transport =
+    object
+      method name = transport#name
+      method protocol = transport#protocol
+      method default_port = transport#default_port
+
+      method connect ?bind_address ?timeout ?prefer =
+        transport#connect ?bind_address ?timeout
+          ~prefer:(Option.value ~default:prefer_address prefer)
+    end
+  in
   let user =
     match (protocol, s_opt "user") with
       | Cry.Http _, None -> "source"
@@ -449,13 +477,12 @@ class output p =
         f (Lang.to_product v))
       (Lang.to_list (List.assoc "headers" p))
   in
-  let transport = (transport :> Cry.transport) in
   let connection = Cry.create ~timeout ~transport ?connection_timeout () in
   object (self)
     inherit
       Output.encoded
-        ~output_kind:"output.icecast" ~infallible ~autostart ~on_start ~on_stop
-          ~name source
+        ~output_kind:"output.icecast" ~infallible ~register_telnet ~autostart
+          ~export_cover_metadata:false ~on_start ~on_stop ~name source
 
     (** In this operator, we don't exactly follow the start/stop
     * mechanism of Output.encoded because we want to control
@@ -520,6 +547,7 @@ class output p =
 
     method icecast_send b =
       try
+        if Cry.get_status connection = Cry.Disconnected then self#icecast_start;
         Strings.iter
           (fun s offset length -> Cry.send connection ~offset ~length s)
           b;
@@ -540,8 +568,7 @@ class output p =
     method send b =
       match Cry.get_status connection with
         | Cry.Disconnected when Unix.time () > restart_time ->
-            self#icecast_start;
-            self#send b
+            self#icecast_send b
         | Cry.Connected _ -> self#icecast_send b
         | _ -> ()
 
