@@ -32,11 +32,19 @@ let encoder_factory ?format format_val =
   with Not_found ->
     raise (Error.Invalid_value (format_val, "Unsupported encoding format"))
 
+let base_proto =
+  ( "export_cover_metadata",
+    Lang.bool_t,
+    Some (Lang.bool true),
+    Some "Export cover metadata." )
+  :: Output.proto
+
 class virtual base ~source ~name p =
   let e f v = f (List.assoc v p) in
   (* Output settings *)
   let autostart = e Lang.to_bool "start" in
   let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
+  let register_telnet = Lang.to_bool (List.assoc "register_telnet" p) in
   let on_start =
     let f = List.assoc "on_start" p in
     fun () -> ignore (Lang.apply f [])
@@ -45,11 +53,14 @@ class virtual base ~source ~name p =
     let f = List.assoc "on_stop" p in
     fun () -> ignore (Lang.apply f [])
   in
+  let export_cover_metadata =
+    Lang.to_bool (List.assoc "export_cover_metadata" p)
+  in
   object (self)
     inherit
       Output.encoded
-        ~infallible ~on_start ~on_stop ~autostart ~output_kind:"output.file"
-          ~name source
+        ~infallible ~register_telnet ~on_start ~on_stop ~autostart
+          ~export_cover_metadata ~output_kind:"output.file" ~name source
 
     val mutable encoder = None
     val mutable current_metadata = None
@@ -97,7 +108,7 @@ class virtual base ~source ~name p =
 (** url output: discard encoded data, try to restart on encoding error (can be
     networking issues etc.) *)
 let url_proto frame_t =
-  Output.proto
+  base_proto
   @ [
       ("url", Lang.string_t, None, Some "Url to output to.");
       ( "restart_delay",
@@ -227,7 +238,7 @@ let default_reopen_when =
     "fun () -> false"
 
 let pipe_proto frame_t arg_doc =
-  Output.proto
+  base_proto
   @ [
       ( "reopen_on_error",
         Lang.fun_t
@@ -359,7 +370,9 @@ class virtual piped_output ~name p =
         (try self#cleanup_pipe with _ -> ());
         self#reopen_on_error ~bt exn
 
-    method! send b =
+    method! send b = if self#is_open then base#send b
+
+    method! encode frame ofs len =
       (match self#is_open with
         | false when open_date <= Unix.gettimeofday () -> self#prepare_pipe
         | true when Atomic.get need_reopen ->
@@ -367,10 +380,9 @@ class virtual piped_output ~name p =
         | true
           when open_date +. reopen_delay () <= Unix.gettimeofday ()
                && reopen_when () ->
-            Atomic.set need_reopen true;
-            open_date <- Unix.gettimeofday ()
+            self#reopen
         | _ -> ());
-      if self#is_open then base#send b
+      base#encode frame ofs len
 
     method! insert_metadata metadata =
       if reopen_on_metadata (Export_metadata.to_metadata metadata) then
