@@ -119,7 +119,7 @@ class once ~name ~timeout request =
         remaining <- decoder.Decoder.fill buf;
         if Frame.is_partial buf then self#end_track false)
 
-    method seek len =
+    method! seek len =
       let decoder = Option.get decoder in
       decoder.Decoder.fseek len
 
@@ -255,7 +255,7 @@ class virtual unqueued ~name =
       in
       if end_track then self#end_track false
 
-    method seek x = match current with None -> 0 | Some cur -> cur.seek x
+    method! seek x = match current with None -> 0 | Some cur -> cur.seek x
     method seek_source = (self :> Source.source)
     method abort_track = self#end_track true
     method! private sleep = self#end_track false
