@@ -9,9 +9,10 @@ class encoded_test =
     inherit
       Output.encoded
         ~output_kind:"foo" ~name:"encoded_test" ~infallible:false
+          ~register_telnet:false
         ~on_start:(fun _ -> ())
         ~on_stop:(fun _ -> ())
-        ~autostart:false
+        ~autostart:false ~export_cover_metadata:false
         (Lang.source (new Noise.noise None))
 
     method insert_metadata _ =
